from dataclasses import dataclass
from datetime import datetime
from typing import List


@dataclass
class Racer:
    """
    A class used to represent Racer.
    Attributes
    ----------
    lap_time
        Time in road.
    car
        Model car.
    driver
        Full driver name.
    """
    lap_time: str
    car: str
    driver: str
    abr: str


class RacingDataAnalyzer:
    """
    Class order racers by time and print report that shows the 15 racers(asc or desc) and the rest after underline.
    """

    def __init__(self, raw_data: List[List[str]]) -> None:
        """
        Parameters
        :param raw_data: text from 3 files
        start_list: list
            Data from start.log
        end_list: list
            Data from end.log
        abbreviations_list: list
            Data from abbreviations.txt
        racer_data: list
            Data from func build_report
        """
        self.sorted_data = None
        self.racer_data = None
        self.start_list, self.end_list, self.abbreviations_list = raw_data

    def build_report(self) -> List[Racer]:
        """
        Lap time calculation.
        :return:
            Data with lap time, car and driver name.
        """
        time_reg = '%H:%M:%S.%f'
        racer_data = []
        for start_item in self.start_list:
            end = str([end_time for end_time in self.end_list if (start_item[0:7] in end_time)])
            abbrev = [name.strip() for name in self.abbreviations_list if (start_item[0:3] in name)]
            lap_time = datetime.strptime(end[16:28], time_reg) - datetime.strptime(start_item[14:26], time_reg)
            if '-' not in str(lap_time):
                abr, driver, car = abbrev[0].split('_', 2)
                racer_data.append(Racer(str(lap_time), car, driver, abr))
        self.racer_data = racer_data
        return self.racer_data

    def print_single_racer(self, driver_name: str) -> None:
        """
        Print driver info.
        Parameters
        :param driver_name:
             Full driver name.
        """
        data_driver = next(iter([driver_info for driver_info in self.racer_data if driver_info.driver == driver_name]))
        print(f"{data_driver.driver}  | {data_driver.car}  | {data_driver.lap_time}")

    def print_reports(self, direction: bool) -> None:
        """
        Print report that shows the 15 racers(asc or desc) and the rest after underline.
        Parameters
        :param direction:
              Shows list of drivers and order by [--asc | --desc].
        """
        sorted_data = sorted(self.racer_data, key=lambda time: time.lap_time, reverse=direction)
        enumerate_data = enumerate(sorted_data, start=1)
        for number, data_driver in enumerate_data:
            if number == 16:  # top 15 racers and the rest after underline
                print('_' * 40)
            print(f"{number}. {data_driver.driver}  | {data_driver.car}  | {data_driver.lap_time}")

    def sort_by_time(self, direction: bool) -> List[Racer]:
        """
        Return list that show racers(asc or desc).
        Parameters
        :param direction:
              Sort list of drivers and order by [--asc | --desc].
        """
        self.sorted_data = sorted(self.racer_data, key=lambda time: time.lap_time, reverse=direction)
        return self.sorted_data

    def enumerate_drivers(self) -> List[Racer]:
        """
        Return report with enumerate.
        """
        return list(enumerate(self.sorted_data, start=1))

    def find_driver_by_code(self, driver_code: str) -> Racer:
        """
        Return driver info.
        Parameters
        :param driver_code
        """
        return next(iter([driver_info for driver_info in self.racer_data if driver_info.abr == driver_code]))



import os
from typing import List


def validate_path(folder_path: str) -> str:
    """Checks if the folder exists."""
    data_folder = os.path.join(os.path.abspath('.'), folder_path)
    if not os.path.isdir(data_folder):
        raise FileNotFoundError('Folder not found')
    return str(data_folder)


def read_files(data_folder: str) -> List[List[str]]:
    """Read files from files and store in lists"""
    data_from_files = []
    files = ["start.log", "end.log", "abbreviations.txt"]
    for file in files:
        file = os.path.join(data_folder, file)
        with open(file) as file_:
            data_from_files.append([line for line in file_ if line.strip() != ""])
    return data_from_files



import argparse
from argparse import Namespace
from typing import Optional

from report_monaco.racing_cli import RacingDataAnalyzer
from report_monaco.utils import read_files, validate_path


def run_parser(args: list = None) -> Optional[Namespace]:
    """Run parser with arguments"""
    parser = argparse.ArgumentParser(description="Race statistics")
    parser.add_argument('--files', dest="folder", help="File folder")
    parser.add_argument('--asc', dest="asc", action='store_const', const=False, help='From slow to faster')
    parser.add_argument('--desc', dest="asc", action='store_const', const=True, help='From faster to slow')
    parser.add_argument('--driver', dest="driver", help='driver name')
    return parser.parse_args(args, namespace=None)


def main() -> None:
    args = run_parser()
    if {None} in [{args.asc, args.driver}, {args.folder}]:
        raise ValueError('Please select --file dir --asc/--desc or --driver')
    folder_path = validate_path(args.folder)
    raw_data = read_files(folder_path)
    analyzer = RacingDataAnalyzer(raw_data)
    analyzer.build_report()
    if args.driver is not None:
        return analyzer.print_single_racer(driver_name=args.driver)
    return analyzer.print_reports(direction=args.asc)
